package com.yixin.ms.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.*;
import cn.hutool.http.HttpUtil;
import com.alibaba.excel.util.DateUtils;
import com.alibaba.fastjson.JSON;
import com.beust.jcommander.internal.Lists;
import com.yixin.ms.boot.restful.RestCode;
import com.yixin.ms.boot.restful.RestResponse;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.boot.uitls.Preconditions;
import com.yixin.ms.boot.uitls.RedisUtil;
import com.yixin.ms.model.dto.HealthCodeDTO;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.enums.*;
import com.yixin.ms.model.vo.HealtCodeNucleinVO;
import com.yixin.ms.model.vo.HealtCodeTypeVO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.*;
import com.yixin.ms.server.dto.*;
import com.yixin.ms.server.vo.*;
import com.yixin.ms.service.HealthCodeService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.nio.charset.Charset;
import java.text.ParseException;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executor;


@Slf4j
@Service
public class HealthCodeServiceImpl implements HealthCodeService {

    @Autowired
    private RedisUtil redisUtil;
    @Autowired
    private PlaceHealthCodeServer placeHealthCodeServer;
    @Autowired
    private HealthCodeServer healthCodeServer;
    @Autowired
    private HealthServer healthServer;
    @Autowired
    private ScenicServer scenicServer;
    @Resource
    private Executor executor;
    @Resource
    private UserBlackServer userBlackServer;
    @Resource
    private HealthOpenReturnServer healthOpenReturnServer;

    @Override
    public HealtCodeUserVO getShortTermCertificate(ShortTermDTO shortTermDTO) {
        return healthServer.getShortTermCertificate(shortTermDTO);
    }

    private HealtCodeUserVO getShortTermCertificate(ShortTermDTO shortTermDTO, PlaceInfoVO placeInfoVO) {
        HealtCodeUserVO userVO = null;
        if (Preconditions.isNotBlank(placeInfoVO)
                && Preconditions.isNotBlank(placeInfoVO.getIsOpenHealthCode())
                && placeInfoVO.getIsOpenHealthCode().intValue() == 1) {//是否开启健康码
            if (Preconditions.isNotBlank(placeInfoVO.getAddressCode())) {
                HealthResultVO<DeviceScanIdCardVO> scanIdCardResult = deviceScanIdCardEncrypt(shortTermDTO, placeInfoVO);
                userVO = scanIdCardResultToHealtCodeUserVO(userVO, scanIdCardResult, shortTermDTO, placeInfoVO);
            }
            if (Preconditions.isBlank(userVO)) {
                userVO = healthServer.getShortTermCertificate(shortTermDTO);
            }
        }
        return userVO;
    }

    @Override
    public String addToken(String token) {
        String s = "hh_token_" + UUID.randomUUID();
        boolean hanghui = redisUtil.set(s, s);
        if (hanghui) {
            return s;
        } else {
            throw new ServiceException("体检失败");
        }
    }

    @Override
    public Boolean deleteToken(String token) {
        return redisUtil.delete(token);
    }

    public static void main(String[] args) {
        String url = "https://h5.dingtalk.com/healthAct/index.html?stateCouncilBarCode=ffd631d12c5ee80fe77fe0fb4347b4e8&flag=1#/result";
        Map<String, String> map = HttpUtil.decodeParamMap(url, Charset.defaultCharset());

        String barCode = HttpUtil.decodeParamMap(url.toLowerCase(), Charset.defaultCharset()).get("statecouncilbarcode");
    }

    @Override
    public RestResult<HealtCodeUserVO> getShortTerm(ShortTermDTO shortTermDTO) {
        log.info("--->联动服务查询入参信息：{}", JSON.toJSONString(shortTermDTO));
        if (StringUtils.isBlank(shortTermDTO.getToken())) {
            throw new ServiceException(RestCodeEnum.FORBIDDEN.getCode(), RestCodeEnum.FORBIDDEN.getMsg());
        }
        /*回参带开门信号*/
        HealtCodeUserVO userVO = null;
        /*查询设备场所码信息   以及设备所绑账号的开关门信息*/
        PlaceInfoVO placeInfoVO = placeHealthCodeServer.queryPlaceInfoBySn(shortTermDTO.getSn());
        //判断是否是考试场景设备
        if (placeInfoVO != null
                && placeInfoVO.getExamDevices() != null) {
            List<String> sns = Arrays.asList(placeInfoVO.getExamDevices().split(","));
            if (sns.contains(shortTermDTO.getSn())) {
                return examBiz(shortTermDTO, placeInfoVO);
            }
        }

        /*换取barCode*/
        if (StrUtil.isNotBlank(shortTermDTO.getHealthCode())) {
            convertHealthCode(shortTermDTO, shortTermDTO.getHealthCode(), placeInfoVO);
        } else {
            if (StringUtils.isBlank(shortTermDTO.getIdNumber()) && StringUtils.isBlank(shortTermDTO.getName())) {
                throw new ServiceException(RestCodeEnum.PARAMS_IDCARD_INCOMPLETE.getCode(), RestCodeEnum.PARAMS_IDCARD_INCOMPLETE.getMsg());
            }
        }

        if (Preconditions.isNotBlank(shortTermDTO.getHealthCode())) {
            //扫二维码
            userVO = getHealthCodeInfoByBarcode(shortTermDTO, placeInfoVO);
            if (Preconditions.isBlank(userVO) && Preconditions.isNotBlank(shortTermDTO.getIdNumber())) {
                //barcode兜底逻辑
                if (Preconditions.isNotBlank(placeInfoVO)
                        && Preconditions.isNotBlank(placeInfoVO.getIsOpenHealthCode())
                        && placeInfoVO.getIsOpenHealthCode().intValue() == 1) { //是否开启健康码
                    userVO = getShortTermCertificate(shortTermDTO);
                }
            }
        } else {
            //人脸识别 刷身份证
            if (Preconditions.isNotBlank(shortTermDTO.getIdNumber())) {
                userVO = getShortTermCertificate(shortTermDTO, placeInfoVO);
            }
        }

        // 根据查询到的信息，返回设备显示信息
        if (Preconditions.isNotBlank(placeInfoVO)
                && (Preconditions.isBlank(placeInfoVO.getIsOpenHealthCode()) ||
                (Preconditions.isNotBlank(placeInfoVO.getIsOpenHealthCode())
                        && placeInfoVO.getIsOpenHealthCode().intValue() == 1))) {
            if (Preconditions.isBlank(userVO) || CollUtil.isEmpty(userVO.getData())) {
                throw new ServiceException(RestCodeEnum.HEALTH_CODE_INFO_NOT_FOUND.getCode(), RestCodeEnum.HEALTH_CODE_INFO_NOT_FOUND.getMsg());
            }
        } else {
            if (Preconditions.isBlank(userVO)) {
                userVO = new HealtCodeUserVO();
            }
        }

        // 姓名身份证数据脱敏处理
        barCodeConvertInfo(userVO, shortTermDTO);
        // 黑名单校验
        Boolean isBlack = userBlackServer.checkUserBlackListURL(userVO.getIdNumber(), shortTermDTO.getSn());
        placeInfoVO.setIsThroughBlackCheck(isBlack);
        // 疫苗 接种数量错误修改  核酸信息修复 健康码信息处理
        healthVaccineNum(userVO, placeInfoVO);
        // 业务返回参数补全 用户核酸信息、开关门校验及语音提示、核酸过期标识
        resultUserVOInfo(userVO, placeInfoVO, shortTermDTO);
        // 游客数据推送 进入方法先校验是否是景区游客并且通行，才会推送记录
        scenicServer.checkConsume(userVO, placeInfoVO, shortTermDTO);
        log.info("开门信号结果返回:{}", JSON.toJSONString(userVO));
        return RestResponse.ok(userVO);
    }

    //考试场景
    private RestResult examBiz(ShortTermDTO shortTermDTO, PlaceInfoVO placeInfoVO) {
        if (StrUtil.isBlankIfStr(shortTermDTO.getIdNumber())) {
            throw new ServiceException(701, "请刷脸或扫描身份证进入");
        }
        HealtCodeUserVO userVO = new HealtCodeUserVO();
        userVO.setIdNumber(StrUtil.isEmpty(shortTermDTO.getIdNumber()) ? StrUtil.EMPTY : shortTermDTO.getIdNumber());
        userVO.setShowIdNumber(StrUtil.isEmpty(shortTermDTO.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(shortTermDTO.getIdNumber(), 4, 4));
        userVO.setUsername(StrUtil.isEmpty(shortTermDTO.getName()) ? StrUtil.EMPTY : shortTermDTO.getName());
        userVO.setShowUsername(StrUtil.isEmpty(shortTermDTO.getName()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(shortTermDTO.getName()));
        //查询
        RestResult<QueryExamUserWhitelistVO> result = placeHealthCodeServer.getExamUserWhitelist(shortTermDTO.getSn(), shortTermDTO.getIdNumber());
        initExamHealthInfo(userVO, placeInfoVO, result);
        return RestResponse.ok(userVO);
    }

    private void initExamHealthInfo(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO, RestResult<QueryExamUserWhitelistVO> result) {
        Integer isOpen = HealthOpenEnum.RELEASE.getCode();
        String codeTypeVOContent = "祝您考试顺利";
        //准考证号
        String travelValidation = "准考证号";
        //场次
        String content = "暂无";
        //报考类别
        String vaccineNum = "暂无";
        if (result != null && result.getData() != null) {
            travelValidation = result.getData().getExamCode();
            content = result.getData().getExamSession();
            vaccineNum = result.getData().getExamType();
        }

        String iconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/green_tick.png";
        String vaccineIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/green_tick.png";
        String travelValidationIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/trip/green.png";
        if (result == null || result.getData() == null) {
            isOpen = HealthOpenEnum.NO_THOROUGHFARE.getCode();
            codeTypeVOContent = "未预约考试，禁止通行";
            iconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/exclamation_mark.png";
            vaccineIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/exclamation_mark.png";
            travelValidationIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/exclamation_mark.png";
        }
        //开门信号
        userVO.setIsOpen(isOpen);
        //语音播报
        userVO.setOpenMsg(codeTypeVOContent != null ? codeTypeVOContent : "祝您考试顺利");
        userVO.setOpenVoiceMsg(codeTypeVOContent != null ? codeTypeVOContent : "祝您考试顺利");

        //全局背景
        userVO.setBackgroundColor(ColorEnum.GREEN.getValue());

        //头部
        String title = "温馨提醒"; //温馨提醒
        if (placeInfoVO != null
                && placeInfoVO.getTopTitle() != null) {
            title = placeInfoVO.getTopTitle();
        }
        userVO.setTitle(title == null ? "温馨提醒" : title);
        userVO.setTitleColor(ColorEnum.WHITE.getValue());

        //中上部
        HealtCodeTypeVO codeTypeVO = new HealtCodeTypeVO();
        codeTypeVO.setMzt(StrUtil.EMPTY);
        codeTypeVO.setContent(codeTypeVOContent);
        codeTypeVO.setContentColor(ColorEnum.GREEN.getValue());
        userVO.setData(Arrays.asList(codeTypeVO));

        //中下部
        HealtCodeNucleinVO v = new HealtCodeNucleinVO();
        //左icon
        v.setIconUrl(iconUrl);
        v.setTitle("场次");
        v.setContent(content).setContentColor(ColorEnum.GREEN.getValue());
        v.setShowTime(null);
        Date date = null;
        String format = DateUtils.format(new Date(), DateUtils.DATE_FORMAT_19);
        try {
            date = DateUtils.parseDate(format, DateUtils.DATE_FORMAT_19);
        } catch (Exception e) {
            e.printStackTrace();
        }
        v.setChecktime(date);
        v.setReportTime(date);
        v.setResult(StrUtil.EMPTY);
        userVO.setNuclein(Arrays.asList(v));

        //右icon
        userVO.setVaccineIconUrl(vaccineIconUrl);
        userVO.setVaccineIcoTitle("报考类别");
        userVO.setVaccineNum(vaccineNum);
        userVO.setVaccineNumColor(ColorEnum.GREEN.getValue());

        //底部
        userVO.setTravelValidationIconUrl(travelValidationIconUrl);
        userVO.setTravelValidationTitle("准考证号");
        userVO.setTravelValidation(travelValidation);
        userVO.setTravelValidationDesc(StrUtil.EMPTY);
    }

    /**
     * 补全返回参数数据
     * 1：用户核酸信息
     * 2：开关门规则校验 及语音提示
     * 3：核酸过期标识
     *
     * @param userVO
     * @param placeInfoVO
     * @param shortTermDTO
     */
    private void resultUserVOInfo(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO, ShortTermDTO shortTermDTO) {
        //用户核酸信息
        List<HealtCodeNucleinVO> nucleinList = userVO.getNuclein();
        //开关门规则校验 及语音提示
        HealthOpenReturnVO healthOpenReturnVO = healthOpenReturnServer.healthOpenReturn(userVO, placeInfoVO, shortTermDTO);
        //核酸过期标识
        Boolean nucleicExpirationFlag = healthOpenReturnVO.getNucleicExpirationFlag();
        if (Preconditions.isNotBlank(placeInfoVO)
                && Preconditions.isNotBlank(placeInfoVO.getIsOpenHealthCode())
                && placeInfoVO.getIsOpenHealthCode().intValue() == 0) { //未开启健康码
            return;
        }

        if (CollectionUtil.isNotEmpty(nucleinList)) {
            HealtCodeNucleinVO nucleinVO = nucleinList.stream().findFirst().get();
            nucleinVO.setTitle("核酸检测").setContentColor(nucleicExpirationFlag ? ColorEnum.YELLOW.getValue() : ColorEnum.GREEN.getValue()).setIconUrl(nucleicExpirationFlag ? NucleicStatusEnum.NOT_AVAILABLE.getUrl() : NucleicStatusEnum.NEGATIVE.getUrl());
            if (nucleinVO.getReportTime() != null && DateUtil.compare(nucleinVO.getChecktime(), nucleinVO.getReportTime()) > 0) {
                nucleinVO.setContent("结果未出");
            } else {
                long betweenHour = DateUtil.between(nucleinVO.getShowTime(), DateUtil.date(), DateUnit.HOUR);
                if (StrUtil.isEmpty(nucleinVO.getResult())) {
                    nucleinVO.setResult("结果未出");
                }
                if (betweenHour < NucleicRuleEnum.TWENTY_FOUR.getValue()) {
                    nucleinVO.setContent(NucleicRuleEnum.TWENTY_FOUR.getValue() + " " + nucleinVO.getResult());
                } else if (betweenHour < NucleicRuleEnum.FORTY_EIGHT.getValue()) {
                    nucleinVO.setContent(NucleicRuleEnum.FORTY_EIGHT.getValue() + " " + nucleinVO.getResult());
                } else if (betweenHour < NucleicRuleEnum.SEVENTY_TWO.getValue()) {
                    nucleinVO.setContent(NucleicRuleEnum.SEVENTY_TWO.getValue() + " " + nucleinVO.getResult());
                } else {
                    nucleinVO.setContent((DateUtil.betweenDay(nucleinVO.getShowTime(), DateUtil.date(), false) + 1) + "天 " + nucleinVO.getResult());
                }
            }
        } else {
            userVO.setNuclein(Lists.newArrayList(new HealtCodeNucleinVO().setTitle("核酸检测").setIconUrl(NucleicStatusEnum.NOT_AVAILABLE.getUrl()).setContent("暂无").setContentColor(ColorEnum.GREEN.getValue())));
        }


        userVO.setTitle("健 康 码").setTitleColor(ColorEnum.WHITE.getValue())
                //疫苗信息
                .setVaccineIcoTitle("新冠疫苗").setVaccineIconUrl(HealthVaccineNumEnum.numOf(userVO.getVaccineNum()).getUrl()).setVaccineNumColor(ColorEnum.GREEN.getValue())
                //行程信息
                .setTravelValidationDesc(TravelValidationEnum.codeOf(userVO.getTravelValidation()).getDesc()).setTravelValidationTitle("动态行程卡").setTravelValidationIconUrl(TravelValidationEnum.codeOf(userVO.getTravelValidation()).getUrl());
    }

    private HealtCodeUserVO getHealthCodeInfoByBarcode(ShortTermDTO shortTermDTO, PlaceInfoVO placeInfo) {

        if (Preconditions.isBlank(placeInfo)) {
            //理论上不会进去这里 设备未注册
            throw new ServiceException(RestCodeEnum.DEVICE_NOT_FOUND.getCode(), RestCodeEnum.DEVICE_NOT_FOUND.getMsg());
        }

        HealtCodeUserVO userVO = null;
        HealthResultVO<DeviceScanIdCardVO> scanIdCardResult = null;
        if (StrUtil.isNotEmpty(shortTermDTO.getHealthCode())) {
            if (NumberUtil.isNumber(shortTermDTO.getHealthCode())) {
                //预约码查询
                //TODO 以后可能就要走策略模式 根据SN获取adminId配置的策略 调用不同的获取用户身份信息的接口
                ScenicVerifyVO verifyVO = scenicServer.verify(new ScenicVerifyDTO().setReservationCode(shortTermDTO.getHealthCode()), placeInfo.getAdminId());
                if (Preconditions.isNotBlank(verifyVO) && !verifyVO.getReleaseStatus()) {
                    //未预约
                    throw new ServiceException(verifyVO.getMessage());
                }
                shortTermDTO.setName(verifyVO.getUserName()).setIdNumber(verifyVO.getIdNumber());
                if (Preconditions.isNotBlank(placeInfo.getAddressCode())
                        && Preconditions.isNotBlank(placeInfo.getIsOpenHealthCode())
                        && placeInfo.getIsOpenHealthCode().intValue() == 1) { //是否开启健康码
                    //场所码
                    scanIdCardResult = deviceScanIdCardEncrypt(shortTermDTO, placeInfo);
                    userVO = scanIdCardResultToHealtCodeUserVO(userVO, scanIdCardResult, shortTermDTO, placeInfo);
                }
            } else if (!NumberUtil.isNumber(shortTermDTO.getHealthCode())) {
                if (Preconditions.isNotBlank(placeInfo.getAddressCode())
                        && Preconditions.isNotBlank(placeInfo.getIsOpenHealthCode())
                        && placeInfo.getIsOpenHealthCode().intValue() == 1) { //开启健康码
                    //健康码查询（四码一屏） 是不返回姓名和身份证号的
                    try {
                        CompletableFuture<HealthResultVO<DeviceScanIdCardVO>> healthResultVOCompletableFuture = CompletableFuture.supplyAsync(() -> deviceScanCode(shortTermDTO, placeInfo));
                        scanIdCardResult = healthResultVOCompletableFuture.get();
                    } catch (Exception e) {
                        e.printStackTrace();
                    }

                    if (Preconditions.isNotBlank(scanIdCardResult) && scanIdCardResult.getCode().intValue() != 200) {
                        throw new ServiceException(scanIdCardResult.getCode(), scanIdCardResult.getMsg());
                    }
                    userVO = scanIdCardResultToHealtCodeUserVO(userVO, scanIdCardResult, shortTermDTO, placeInfo);
                }

                //barcode换取用户身份信息 身份证和姓名 未脱敏
                CompletableFuture<HealthCodeVO> HealthCodeVOCompletableFuture = CompletableFuture.supplyAsync(() -> healthServer.queryInfoByBarCode(shortTermDTO.getHealthCode()));
                try {
                    HealthCodeVO healthCodeVO = HealthCodeVOCompletableFuture.get();
                    if (Preconditions.isNotBlank(healthCodeVO)) {
                        shortTermDTO.setName(healthCodeVO.getUserName()).setIdNumber(healthCodeVO.getIdNum());
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }

            }
        } else {
            throw new ServiceException(RestCodeEnum.INVALID_QR_CODE.getCode(), RestCodeEnum.INVALID_QR_CODE.getMsg());
        }
        return userVO;
    }

    private HealtCodeUserVO scanIdCardResultToHealtCodeUserVO(HealtCodeUserVO userVO, HealthResultVO<DeviceScanIdCardVO> scanIdCardResult, ShortTermDTO shortTermDTO, PlaceInfoVO placeInfoVO) {
        if (scanIdCardResult != null && scanIdCardResult.getCode().intValue() == RestCode.SUCCESS.getCode()) {
            DeviceScanIdCardVO scanIdCardVO = scanIdCardResult.getData();
            /*数据转换*/
            userVO = deviceScanIdCardVOConvert(scanIdCardVO, shortTermDTO);
            /*场所码数据推送*/
            if (Preconditions.isNotBlank(placeInfoVO)
                    && Preconditions.isNotBlank(placeInfoVO.getAddressCode())) {
                savePlaceRecord(shortTermDTO, scanIdCardVO, placeInfoVO);
            }
        }
        return userVO;
    }


    private HealtCodeUserVO deviceScanIdCardVOConvert(DeviceScanIdCardVO scanIdCardVO, ShortTermDTO shortTermDTO) {
        HealtCodeUserVO userVO = new HealtCodeUserVO();
        //疫苗接种数量
        userVO.setVaccineNum(scanIdCardVO.getVaccinated());
        //核酸记录
        if (StrUtil.isNotEmpty(scanIdCardVO.getCheckTime())) {
            userVO.setNuclein(Lists.newArrayList(new HealtCodeNucleinVO().setChecktime(DateUtil.parseDateTime(scanIdCardVO.getCheckTime())).setReportTime(StrUtil.isEmpty(scanIdCardVO.getReportTime()) ? null : DateUtil.parseDateTime(scanIdCardVO.getReportTime())).setResult(scanIdCardVO.getCheckResult())));
        } else {
            userVO.setNuclein(Lists.newArrayList());
        }
        //健康码信息
        userVO.setData(Lists.newArrayList(new HealtCodeTypeVO().setMzt(HealthCodeLevelEnum.codeOf(scanIdCardVO.getLevel()).getDesc())));
        //行程信息
        userVO.setTravelValidation(scanIdCardVO.getTravelValidation());
        userVO.setUsername(StrUtil.isEmpty(shortTermDTO.getName()) ? scanIdCardVO.getUsername() : shortTermDTO.getName());
        userVO.setLimitGather(Preconditions.isNotBlank(scanIdCardVO.getLimitGather()) ? scanIdCardVO.getLimitGather() : Boolean.FALSE);
        return userVO;
    }

    private void savePlaceRecord(ShortTermDTO shortTermDTO, DeviceScanIdCardVO scanIdCardVO, PlaceInfoVO placeInfoVO) {
        PlaceRecordDTO placeRecordDTO = new PlaceRecordDTO().setName(StrUtil.isEmpty(shortTermDTO.getName()) ? scanIdCardVO.getUsername() : shortTermDTO.getName()).setIdNumber(shortTermDTO.getIdNumber()).setLevel(HealthCodeLevelEnum.codeOf(scanIdCardVO.getLevel()).getDesc())
                //检测时间
                .setCheckTime(scanIdCardVO.getCheckTime() == null ? null : DateUtil.parseDateTime(scanIdCardVO.getCheckTime()))
                //报告时间
                .setReportTime(scanIdCardVO.getReportTime() == null ? null : DateUtil.parseDateTime(scanIdCardVO.getReportTime()))
                //检测结果
                .setCheckResult(scanIdCardVO.getCheckResult())
                .setTravelValidation(scanIdCardVO.getTravelValidation())
                .setVaccinated(scanIdCardVO.getVaccinated())
                .setAddressCode(placeInfoVO.getAddressCode())
                .setUnitName(placeInfoVO.getUnitName())
                .setUnitAddress(placeInfoVO.getUnitAddress())
                .setManagerHealthCode(placeInfoVO.getManagerHealthCode())
                .setAdminId(placeInfoVO.getAdminId())
                .setHealthCode(shortTermDTO.getHealthCode())
                .setCreateTime(DateUtil.date());
        executor.execute(() -> {
            //保存场所记录
            placeHealthCodeServer.savePlaceRecord(placeRecordDTO);
        });
    }

    /**
     * 处理并提取healthCode
     *
     * @param healthCodeSource 000026https://h5.dingtalk.com/healthAct/index.html?stateCouncilBarCode=ffd631d12c5ee80fe77fe0fb4347b4e8&flag=1#/result
     *                         扫码参数兼容问题处理
     */
    private void convertHealthCode(ShortTermDTO shortTermDTO, String healthCodeSource, PlaceInfoVO placeInfoVO) {

        if (Preconditions.isNotBlank(placeInfoVO)
                && Preconditions.isNotBlank(placeInfoVO.getIsOpenHealthCode())
                && placeInfoVO.getIsOpenHealthCode().intValue() == 0
                && (shortTermDTO.getHealthCode().contains("http") || shortTermDTO.getHealthCode().contains("HTTP"))) {
            throw new ServiceException(RestCodeEnum.INVALID_QR_CODE.getCode(), "健康码已关闭，请扫描正确的二维码");
        }

        if (!shortTermDTO.getHealthCode().contains("http")
                && !shortTermDTO.getHealthCode().contains("HTTP") && !NumberUtil.isNumber(shortTermDTO.getHealthCode())) {
            throw new ServiceException(RestCodeEnum.INVALID_HEALTH_CODE.getCode(), RestCodeEnum.INVALID_HEALTH_CODE.getMsg());
        }

        String barCode = null;
        //获取url中的barCode信息
        if (shortTermDTO.getHealthCode().contains("http") || shortTermDTO.getHealthCode().contains("HTTP")) {
            if (shortTermDTO.getHealthCode().contains("http")) {
                barCode = HttpUtil.decodeParamMap(healthCodeSource.substring(healthCodeSource.indexOf("http")).toLowerCase(), Charset.defaultCharset()).get("statecouncilbarcode");
                if (Preconditions.isNotBlank(placeInfoVO.getPlaceAreaCode())
                        && PlaceAreaEnum.getByPlaceAreaCode(placeInfoVO.getPlaceAreaCode()) != PlaceAreaEnum.ZHE_JIANG) {
                    barCode = HttpUtil.decodeParamMap(healthCodeSource.substring(healthCodeSource.indexOf("http")), Charset.defaultCharset()).get("qrCode").replace("#/result", "");
                }
            }

            if (shortTermDTO.getHealthCode().contains("HTTP")) {
                barCode = HttpUtil.decodeParamMap(healthCodeSource.substring(healthCodeSource.indexOf("HTTP")).toLowerCase(), Charset.defaultCharset()).get("statecouncilbarcode");
                if (Preconditions.isNotBlank(placeInfoVO.getPlaceAreaCode())
                        && PlaceAreaEnum.getByPlaceAreaCode(placeInfoVO.getPlaceAreaCode()) != PlaceAreaEnum.ZHE_JIANG) {
                    barCode = HttpUtil.decodeParamMap(healthCodeSource.substring(healthCodeSource.indexOf("HTTP")), Charset.defaultCharset()).get("qrCode").replace("#/result", "");
                }
            }
            shortTermDTO.setHealthCode(barCode);
        }

        if (StrUtil.isBlank(shortTermDTO.getHealthCode())) {
            throw new ServiceException(RestCodeEnum.INVALID_HEALTH_CODE.getCode(), RestCodeEnum.INVALID_HEALTH_CODE.getMsg());
        }
    }

    /**
     * 根据barCode查询健康码信息
     *
     * @param shortTermDTO
     * @return
     */
    private HealthResultVO<DeviceScanIdCardVO> deviceScanCode(ShortTermDTO shortTermDTO, PlaceInfoVO placeInfo) {
        return placeHealthCodeServer.deviceScanCode(shortTermDTO, placeInfo);

    }

    /**
     * 根据证件号查询健康码信息
     *
     * @param shortTermDTO
     * @return
     */
    private HealthResultVO<DeviceScanIdCardVO> deviceScanIdCardEncrypt(ShortTermDTO shortTermDTO, PlaceInfoVO placeInfo) {
        return placeHealthCodeServer.deviceScanIdCardEncrypt(shortTermDTO, placeInfo);
    }

    @Override
    public RestResult<HealthCodeVO> queryInfoByBarCode(HealthCodeDTO healthCodeDTO) {
        IrsTripAuthEnum authEnum = IrsTripAuthEnum.appKeyOf(healthCodeDTO.getAppKey());
        if (authEnum == null || !authEnum.getAppSecret().equals(healthCodeDTO.getAppSecret())) {
            return RestResponse.error("调用权限校验失败，请检查");
        }
        return healthCodeServer.queryInfoByBarCode(healthCodeDTO.getBarCode());
    }

    /*在有barCode情况下获取个人信息回填*/
    private void barCodeConvertInfo(HealtCodeUserVO userVO, ShortTermDTO shortTermDTO) {
        //姓名和身份证号脱敏处理
        //姓名 通过barcode获取真实姓名
        if (StrUtil.isNotEmpty(shortTermDTO.getHealthCode())
                && !NumberUtil.isNumber(shortTermDTO.getHealthCode())) {
            HealthCodeVO healthCodeVO = healthServer.queryInfoByBarCode(shortTermDTO.getHealthCode());
            if (healthCodeVO != null) {
                if (healthCodeVO.getUserName() != null) {
                    userVO.setUsername(healthCodeVO.getUserName());
                }

                if (StrUtil.isNotEmpty(healthCodeVO.getPn())) {
                    userVO.setPhone(healthCodeVO.getPn());
                }

                if (healthCodeVO.getIdNum() != null) {
                    userVO.setIdNumber(healthCodeVO.getIdNum());
                } else {
                    //如果返回为null那就设置barcode给idNumber
                    userVO.setIdNumber(shortTermDTO.getHealthCode());
                }
            } else {
                //如果返回为null那就设置barcode给idNumber
                userVO.setIdNumber(shortTermDTO.getHealthCode());
            }
        }

        userVO.setUsername(StrUtil.isEmpty(userVO.getUsername()) ? shortTermDTO.getName() : userVO.getUsername());
        userVO.setIdNumber(StrUtil.isEmpty(userVO.getIdNumber()) ? shortTermDTO.getIdNumber() : userVO.getIdNumber());
        userVO.setShowIdNumber(StrUtil.isEmpty(userVO.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(userVO.getIdNumber(), 4, 4));
        userVO.setShowUsername(StrUtil.isEmpty(userVO.getUsername()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(userVO.getUsername()));
    }

    /**
     * 1.疫苗接种数量修缮
     * 2.核酸信息处理
     * 3.健康码信息处理
     *
     * @param userVO
     */
    private void healthVaccineNum(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO) {
        /**
         * 因为返回参数和针数不是一一对应所以这里做处理，修正
         * 0 未接种 对应0针
         * 1-接种完成 对应2针
         * 2-未完全接种 对应1针
         * 3- 完成加强针 对应3针
         */
        if (Preconditions.isNotBlank(placeInfoVO)
                && Preconditions.isNotBlank(placeInfoVO.getIsOpenHealthCode())
                && placeInfoVO.getIsOpenHealthCode().intValue() == 0) {//未开启健康码
            initHealthVaccineNum(userVO, placeInfoVO);
            return;
        }
        HealtCodeTypeVO codeTypeVO = userVO.getData().stream().findFirst().get();
        if (HealthVaccineNumEnum.VACCINATION_COMPLETE.getStatus().equals(userVO.getVaccineNum())) {
            userVO.setVaccineNum(HealthVaccineNumEnum.VACCINATION_COMPLETE.getNum());
        } else if (HealthVaccineNumEnum.INCOMPLETE_INOCULATION.getStatus().equals(userVO.getVaccineNum())) {
            userVO.setVaccineNum(HealthVaccineNumEnum.INCOMPLETE_INOCULATION.getNum());
        } else {
            userVO.setVaccineNum(userVO.getVaccineNum());
        }
        // 核酸信息处理
        List<HealtCodeNucleinVO> nucleinList = userVO.getNuclein();
        if (CollectionUtil.isNotEmpty(nucleinList)) {
            nucleinList.forEach(v -> {
                Date date = v.getReportTime() == null ? v.getChecktime() : DateUtil.compare(v.getChecktime(), v.getReportTime()) > 0 ? v.getChecktime() : v.getReportTime();
                v.setShowTime(date);
                v.setChecktime(v.getChecktime());
                v.setReportTime(v.getReportTime());
            });
        }
        // 健康码处理
        if (HealthCodeLevelEnum.GREEN.getDesc().equals(codeTypeVO.getMzt())) {
            codeTypeVO.setContent("绿码：健康码状态为低风险").setContentColor(ColorEnum.GREEN.getValue());
            userVO.setBackgroundColor(ColorEnum.GREEN.getValue());
        }
    }

    private void initHealthVaccineNum(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO) {

        //验证方式
        String vaccineNum = "身份证验证";
        //全局背景色
        userVO.setBackgroundColor(ColorEnum.BLUE.getValue());
        //头部
        String title = "温馨提示";
        if (Preconditions.isNotBlank(placeInfoVO)
                && Preconditions.isNotBlank(placeInfoVO.getTopTitle())) {
            title = placeInfoVO.getTopTitle();
        }
        userVO.setTitle(title);
        userVO.setTitleColor(ColorEnum.WHITE.getValue());
        //中上部
        userVO.setVaccineIconUrl("https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/pass.png");
        userVO.setVaccineNumColor("#2b6ef5");
        //中下左部
        HealtCodeNucleinVO v = new HealtCodeNucleinVO();
        v.setShowTime(null);
        Date date = null;
        String format = DateUtils.format(new Date(), DateUtils.DATE_FORMAT_19);
        try {
            date = DateUtils.parseDate(format, DateUtils.DATE_FORMAT_19);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        v.setChecktime(date);
        v.setReportTime(date);
        v.setIconUrl("https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/pass.png");
        v.setTitle("温馨提示");
        v.setContent("携带好随身物品").setContentColor("#4e5c74");
        v.setResult(StrUtil.EMPTY);
        userVO.setNuclein(Arrays.asList(v));
        //中下右部
        userVO.setVaccineIcoTitle("验证方式");
        userVO.setVaccineNum(vaccineNum);
        //底部
        HealtCodeTypeVO codeTypeVO = new HealtCodeTypeVO();
        codeTypeVO.setMzt("绿码");
        codeTypeVO.setContent("请通行");
        codeTypeVO.setContentColor(ColorEnum.GREEN.getValue());
        userVO.setData(Arrays.asList(codeTypeVO));

        userVO.setTravelValidationTitle("动态行程卡");
        userVO.setTravelValidationIconUrl("https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/pass.png");
        userVO.setTravelValidationDesc(null);

    }

}

